﻿Type=Class
Version=4.7
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Notes
' Server WebSocket Class Module: LEDSwitchPage
' Purpose:
' Manage browser connection and switching of LED
#End Region

'
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore	
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "FlowPage"
	' name of the app, same as in ABMApplication
	Public AppName As String = "dnc-node"	
	' own project vars
	Dim LabelSwitched As ABMLabel
	Dim SwitchCounter As Int = 0	
	' will hold the unique browsers window id
	Private ABMPageId As String = ""
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	BuildPage
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	Log("WebSocket Connected")
	ws = WebSocket1	
	ABMPageId = ABM.GetPageID(page, Name,ws)
	
	If ABMShared.NeedsAuthorization Then
		If ws.Session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws, ABMPageId,"../")
			Return
		End If
	End If
	' connect our page with the websocket	
	page.SetWebSocket(ws)	
	' Prepare the page IMPORTANT!
	page.Prepare	
	
	ConnectPage
End Sub

Private Sub WebSocket_Disconnected
	Log("WebSocket Disconnected")
End Sub

Sub Page_ParseEvent(Params As Map) 
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	If SubExists(Me, eventName) Then
		Params.Remove("eventname")
		Params.Remove("eventparams")
		Select Case Params.Size
			Case 0
				CallSub(Me, eventName)
			Case 1
				CallSub2(Me, eventName, Params.Get(eventParams(0)))					
			Case 2
				If Params.get(eventParams(0)) = "abmistable" Then
					Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
					CallSub2(Me, eventName, PassedTables)
				Else
					CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
				End If
			Case Else
				' cannot be called diretly, to many param
				CallSub2(Me, eventName, Params)				
		End Select
	End If
End Sub

public Sub BuildTheme()
	' start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)
	
	' add additional themes specific for this page
End Sub

public Sub BuildPage()
	' initialize the theme
	BuildTheme
	
	' initialize this page using our theme
	page.InitializeWithTheme(Name, "/ws/" & AppName & "/" & Name, False,3, theme)
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = "製程"
	page.PageDescription = "B4J Server Application to switch an LED connected to a Raspberry Pi GPIO using ABMaterial."
	page.PageKeywords = "IoT, B4J"
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
	page.ShowConnectedIndicator = True
	ABMShared.BuildNavigationBar(page, "About","../images/flow.png", "", "About", "")	
	' adding a navigation bar
	
	' create the page grid
	' row 1, 1 cell: header
	page.AddRows(1,False, "headertheme").AddCells12(1,"headertheme")
	' row 2, 3 cells: switch + switched info
	page.AddRowsM(1,True,0,0,"").AddCellsOS(2,0,0,0,2,2,2,"")
	' row 3, 1 cell: empty
	page.AddRows(1, True, "").AddCells12(1, "")
	' row 4, 1 cell: divider
	page.AddRows(1, True, "").AddCells12(1, "")
	' row 5, 1 cells: info
	page.AddRows(1, True, "").AddCells12(1, "")
	'
	page.BuildGrid 'IMPORTANT loaded the complete grid prior adding components

	' Create controls - IMPORTANT = EACH CONTROL MUST HAVE AN UNIQUE ID
	' Row1,Cell1 = Add label as header
	Dim lblHeader As ABMLabel
	lblHeader.Initialize(page, "lblHeader", "單機稼動及數據",ABM.SIZE_H4, True, "whitefc")
	lblHeader.VerticalAlign = True
	page.Cell(1,1).AddComponent(lblHeader)

	' Row2,Cell1 = Add switch; defined as global becaused used in the switch event

	' Row2,Cell2 = Add label

	' Row3 = Empty

	' Row4 = Add hor divider
	Dim div1 As ABMDivider
	div1.Initialize(page, "div1", "lightblue")
	page.Row(4).SetDivider(div1)

	' Row5,Cell1 = Add card with connection info
	Dim card1 As ABMCard
	card1.InitializeAsPanel(page, _
							"card1", _
							$"
							{B}Information{/B}
							{BR}The LED is connected to physical pin 15, which is GPIO22 (GPIO_GEN3).
							{BR}In B4J, the LED is assigned to pin 3 as digital output.
							"$, _
							"whitetitle")
	page.Cell(5,1).AddComponent(card1)
	'
	' create list
	Dim list1 As ABMList
	list1.Initialize(page, "list1", ABM.COLLAPSE_ACCORDION, "list1theme")
	' add items
	If dcu.EquipCount>0 Then
		Dim s1,s2,s3,s4 As String
		' 讀取製程
		For n1=1 To dcu.EquipCount
			s4 = dcu.equips.GetKeyAt(n1-1) ' machine name
			Dim mp As Map=dcu.equips.GetValueAt(n1-1)
			s1 = mp.Get("flow") ' flow id
			s2 =	dcu.FlowName(s1) ' flow name
			s3 = mp.Get("no") ' machine id			
			list1.AddItem("H"&s1,BuildSimpleItem0("L1H"&s1,"mdi-hardware-memory",s2))
		Next
		' 讀取機台
		For n1=1 To dcu.EquipCount
			s4 = dcu.equips.GetKeyAt(n1-1) ' machine name
			Dim mp As Map=dcu.equips.GetValueAt(n1-1)
			s1 = mp.Get("flow") ' flow id
			s2 =	dcu.FlowName(s1) ' flow name
			s3 = mp.Get("no") ' machine id			
			list1.AddSubItem("H"&s1, "S"&s3, BuildSimpleItem1("L1H"&s1&"S"&s3,"mdi-action-extension", s4)) 		
		Next
	Else
		list1.AddItem("H1",BuildSimpleItem0("L1H1","mdi-action-account-circle","未設定"))	
	End If
	page.Cell(2,1).AddComponent(list1)
	' Fixed Footer
	ABMShared.BuildFooterFixed(page)
End Sub

' the list clicked event
Sub list1_Clicked(ItemId As String)
   If ItemId.StartsWith("H") Then Return
   Dim list1 As ABMList = page.Component("list1")
   Dim lbl As ABMLabel = list1.Item(ItemId)
   Dim myTexts, myReturns As List
   myTexts.Initialize
   myReturns.Initialize
   page.ShowToast("toast1", "toastred", "Clicked on " & lbl.Text, 5000)
End Sub

' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)
	page.SaveNavigationBarPosition
	If Action = "ABMModalSheet" Then Return
	If Action = "Contact" Then	
		page.ShowToast("toast", "toastred", "Hello to you too!", 5000)
		Return
	End If
	If Action = "LogOff" Then
		ABMShared.LogOff(page)
		Return
	End If
	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub

' When the download is finished, you'll receive an FileUploaded() event on the page.
Sub Page_FileUploaded(FileName As String, success As Boolean)	
	
End Sub

Sub Page_ToastClicked(ToastId As String, Action As String)
		
End Sub

Sub Page_ToastDismissed(ToastId As String)	
	
End Sub

Sub Page_Ready()
	page.RestoreNavigationBarPosition	
   	page.Refresh
	ConnectPage
End Sub

Sub ConnectPage()
	ABMShared.ConnectNavigationBar(page)
End Sub

#Region Buttons
 




#End Region

#Region User Function
Sub BuildSimpleItem0(id As String, icon As String, Title As String) As ABMLabel
	Dim lbl As ABMLabel
	If icon <> "" Then
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "header")
	Else
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "")
	End If
	lbl.VerticalAlign = True
	lbl.IconName = icon
	Return lbl
End Sub

Sub BuildSimpleItem1(id As String, icon As String, Title As String) As ABMLabel
	Dim lbl As ABMLabel
	If icon <> "" Then
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "header")
	Else
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "")
	End If
	lbl.VerticalAlign = True	
	lbl.IconName = icon
	Return lbl
End Sub
#End Region

