﻿Type=Class
Version=4.98
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Notes
' Server WebSocket Class Module: LEDSwitchPage
' Purpose:
' Manage browser connection and switching of LED
#End Region

'
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore	
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "NewNdPage"
	' name of the app, same as in ABMApplication
	Public AppName As String = "dnc-node"
	' will hold the unique browsers window id
	Private ABMPageId As String = ""
	
	' own project vars
	Dim LogTimer As Timer
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	BuildPage
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	Log("WebSocket Connected")
	ws = WebSocket1	
	ABMPageId = ABM.GetPageID(page, Name,ws)	
	
	If ABMShared.NeedsAuthorization Then
		If ws.Session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws,ABMPageId, "../")
			Return
		End If
	End If
	
	' connect our page with the websocket	
	page.SetWebSocket(ws)	

	' Prepare the page IMPORTANT!
	page.Prepare		
	ConnectPage
	LogTimer.Initialize("LogTimer", 3000)
	LogTimer.Enabled = False	
End Sub

Private Sub WebSocket_Disconnected
	Log("WebSocket Disconnected")
	LogTimer.Enabled = False
End Sub

Sub Page_ParseEvent(Params As Map) 
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	If SubExists(Me, eventName) Then
		Params.Remove("eventname")
		Params.Remove("eventparams")
		Select Case Params.Size
			Case 0
				CallSub(Me, eventName)
			Case 1
				CallSub2(Me, eventName, Params.Get(eventParams(0)))					
			Case 2
				If Params.get(eventParams(0)) = "abmistable" Then
					Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
					CallSub2(Me, eventName, PassedTables)
				Else
					CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
				End If
			Case Else
				' cannot be called diretly, to many param
				CallSub2(Me, eventName, Params)				
		End Select
	End If
End Sub

public Sub BuildTheme()
	' start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)
	
	' add additional themes specific for this page
End Sub

public Sub BuildPage()
	' initialize the theme
	Dim s1 As String=page.Handler
	BuildTheme		
	
	' initialize this page using our theme
	page.InitializeWithTheme(Name, "/ws/" & AppName & "/" & Name, False,3, theme)
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = "DNC-Node"
	page.PageDescription = "B4J Server Application to switch an LED connected to a Raspberry Pi GPIO using ABMaterial."
	page.PageKeywords = "IoT, B4J"
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
	page.ShowConnectedIndicator = True
	ABMShared.BuildNavigationBar(page, "新增NODE","../images/logo.png","","Node","Add")
	' adding a navigation bar
	page.AddRows(1, True,"").AddCells12(1,"")
	page.AddRows(1, True,"").AddCells12(1,"")
	page.AddRows(1, True,"").AddCells12(1,"")
	page.AddRows(1, True,"").AddCells12(1,"")
	page.AddRows(1, True,"").AddCells12(1,"")
	page.AddRows(1, True,"").AddCells12(1,"")
	page.BuildGrid 'IMPORTANT loaded the complete grid prior adding components	
	'
	' Fixed Footer
	ABMShared.BuildFooterFixed(page)
End Sub

Sub BuildItem(id As String, text As String, messages As String, isnew As Boolean) As ABMContainer
   Dim ItemCont As ABMContainer
	ItemCont.Initialize(page, id, "")
   ItemCont.AddRowsM(1,False,0,0,"").AddCellsOSMP(1,0,0,0,4,4,4,6,0,0,0,"").AddCellsOSMP(1,0,0,0,8,8,8,6,0,0,0,"")
   ItemCont.BuildGrid ' IMPORTANT once you loaded the complete grid And before you start adding components

   Dim lbl As ABMLabel
   lbl.Initialize(page, id & "lbl", text, ABM.SIZE_PARAGRAPH, False, "")
   lbl.Text=text
   ItemCont.Cell(1,1).AddComponent(lbl)
   
   Dim lbl1 As ABMLabel
   lbl1.Initialize(page, id & "val", text, ABM.SIZE_H5, False, "")
   lbl1.Text = messages
   ItemCont.Cell(1,2).AddComponent(lbl1)
   
   Return ItemCont
End Sub

' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)	
	page.SaveNavigationBarPosition
	If Action = "ABMModalSheet" Then Return
	If Action = "Contact" Then	
		page.ShowToast("toast", "toastred", "Hello to you too!", 5000)
		Return
	End If
	If Action = "Logoff" Then
		ABMShared.LogOff(page)
		Return
	End If
	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub
' When the download is finished, you'll receive an FileUploaded() event on the page.
Sub Page_FileUploaded(FileName As String, success As Boolean)	
	
End Sub

Sub Page_ToastClicked(ToastId As String, Action As String)
		
End Sub

Sub Page_ToastDismissed(ToastId As String)	
	
End Sub

Sub Page_Ready()		   	
	LogTimer.Enabled = True
End Sub

Sub ConnectPage()
	ABMShared.ConnectNavigationBar(page)	
	' NEW, because we use ShowLoaderType=ABM.LOADER_TYPE_MANUAL
	page.FinishedLoading 'IMPORTANT
	page.Refresh
	page.RestoreNavigationBarPosition	
End Sub

Sub LogTimer_Tick
	LogTimer.Interval=1000
	
End Sub
#Region Buttons

#End Region

#Region User Function
Sub BuildSimpleItem0(id As String, icon As String, Title As String) As ABMLabel
	Dim lbl As ABMLabel
	If icon <> "" Then
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "header")
	Else
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "")
	End If
	lbl.VerticalAlign = True
	lbl.IconName = icon
	Return lbl
End Sub

Sub BuildSimpleItem1(id As String, icon As String, Title As String) As ABMLabel
	Dim lbl As ABMLabel
	If icon <> "" Then
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "header")
	Else
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "")
	End If
	lbl.VerticalAlign = True	
	lbl.IconName = icon
	Return lbl
End Sub
#End Region

